@@ -1,23 +1,20 @@
 package Apache::LogRegex;
-{
-  $Apache::LogRegex::VERSION = '1.52';
-}
-
+$Apache::LogRegex::VERSION = '1.60';
 use strict;
 use warnings;
 
 sub new {
-    my ( $class, $format ) = @_;
+    my ($class, $format) = @_;
 
-	die "Apache::LogRegex->new() takes 1 argument" if scalar(@_) != 2;
-	die "Apache::LogRegex->new() argument 1 (FORMAT) is undefined" if !defined($format);
+    die __PACKAGE__ . '->new() takes 1 argument' unless @_ == 2;
+    die __PACKAGE__ . '->new() argument 1 (FORMAT) is false' unless $format;
 
     my $self = bless {}, $class;
 
     $self->{_format} = $format;
 
-    $self->{_regex_string} = '';
-    $self->{_regex_fields} = ();
+    $self->{_regex} = '';
+    $self->{_regex_fields} = undef;
 
     $self->_parse_format();
 
@@ -27,83 +24,110 @@ sub new {
 sub _parse_format {
     my ($self) = @_;
 
-    chomp( $self->{_format} );
+    chomp $self->{_format};
     $self->{_format} =~ s#[ \t]+# #;
     $self->{_format} =~ s#^ ##;
     $self->{_format} =~ s# $##;
 
     my @regex_elements;
 
-    foreach my $element ( split ( ' ', $self->{_format} ) ) {
-        my $quotes = ( $element =~ m/^\\\"/ ) ? 1 : 0;
+    for my $element (split ' ', $self->{_format}) {
+        my $quotes = $element =~ m/^\\\"/ ? 1 : 0;
 
         if ($quotes) {
             $element =~ s/^\\\"//;
             $element =~ s/\\\"$//;
         }
 
-        push ( @{ $self->{_regex_fields} }, $self->rename_this_name($element) );
+        push @{ $self->{_regex_fields} }, $self->rename_this_name($element);
 
         my $x = '(\S*)';
 
         if ($quotes) {
-			if($element eq '%r' or $element =~ m/{Referer}/ or $element =~ m/{User-Agent}/) {
-				$x = qr/"([^"\\]*(?:\\.[^"\\]*)*)"/;
-			}
-			else {
-            	$x = '\"([^\"]*)\"';
-			}
+            if ($element eq '%r' or $element =~ m/{Referer}/ or $element =~ m/{User-Agent}/) {
+                $x = qr/"([^"\\]*(?:\\.[^"\\]*)*)"/;
+            }
+            else {
+                $x = '\"([^\"]*)\"';
+            }
         }
-        elsif ( $element =~ m/^%.*t$/ ) {
+        elsif ($element =~ m/^%.*t$/) {
             $x = '(\[[^\]]+\])';
         }
-		elsif ( $element eq '%U' ) {
-			$x = '(.+?)';
-		}
+        elsif ($element eq '%U') {
+            $x = '(.+?)';
+        }
 
-        push ( @regex_elements, $x );
+        push @regex_elements, $x;
     }
 
-	my $regex = join ( ' ', @regex_elements );
-	$self->{_regex_string} = qr/^$regex\s*$/;
+    my $regex_string = join ' ', @regex_elements;
+    $self->{_regex} = qr/^$regex_string\s*$/;
 }
 
 sub parse {
-    my ( $self, $line ) = @_;
+    my ($self, $line) = @_;
 
-	die "Apache::LogRegex->parse() takes 1 argument" if scalar(@_) != 2;
-	die "Apache::LogRegex->parse() argument 1 (LINE) is undefined" if !defined($line);
+    die __PACKAGE__ . '->parse() takes 1 argument' unless @_ == 2;
+    die __PACKAGE__ . '->parse() argument 1 (LINE) is undefined' unless defined $line;
 
-    chomp($line);
+    my @temp = $line =~ $self->{_regex};
 
-    my @temp = $line =~ m/$self->{_regex_string}/;
-
-    return if scalar(@temp) == 0;
+    return unless @temp;
 
     my %data;
     @data{ @{ $self->{_regex_fields} } } = @temp;
 
-    return %data;
+    return wantarray ? %data : \%data;
+}
+
+sub generate_parser {
+    my ($self, %args) = @_;
+
+    my $regex = $self->{_regex};
+    my @fields = @{ $self->{_regex_fields} };
+
+    no warnings 'uninitialized';
+
+    if ($args{reuse_record}) {
+        my $record = {};
+        return sub {
+            if (@$record{@fields} = $_[0] =~ $regex) {
+                return $record;
+            } else {
+                return;
+            }
+        }
+    } else {
+        return sub {
+            my $record = {};
+            if (@$record{@fields} = $_[0] =~ $regex) {
+                return $record;
+            } else {
+                return;
+            }
+        }
+    }
 }
 
 sub names {
     my ($self) = @_;
 
-	die "Apache::LogRegex->names() takes no argument" if scalar(@_) != 1;
+    die __PACKAGE__ . '->names() takes no argument' unless @_ == 1;
 
-    return ( @{ $self->{_regex_fields} } );
+    return @{ $self->{_regex_fields} };
 }
 
 sub regex {
     my ($self) = @_;
 
-	die "Apache::LogRegex->regex() takes no argument" if scalar(@_) != 1;
+    die __PACKAGE__ . '->regex() takes no argument' unless @_ == 1;
 
-    return $self->{_regex_string};
+    return $self->{_regex};
 }
 
 sub rename_this_name {
-    my ( $self, $name ) = @_;
+    my ($self, $name) = @_;
 
     return $name;
 }
@@ -116,7 +140,7 @@ Apache::LogRegex - Parse a line from an Apache logfile into a hash
 
 =head1 VERSION
 
-version 1.52
+version 1.60
 
 =head1 SYNOPSIS
 
@@ -138,6 +162,15 @@ version 1.52
       }
   }
 
+  # or generate a closure for better performance
+
+  my $parser = $lr->generate_parser;
+
+  while ( my $line_from_logfile = <> ) {
+      my $data = $parser->($line_from_logfile) or last;
+      # We have data to process
+  }
+
 =head1 DESCRIPTION
 
 =head2 Overview
@@ -179,6 +212,18 @@ return a hash of all the elements of the line indexed by their
 format. If the line cannot be parsed an empty hash will be
 returned.
 
+=item generate_parser( LIST )
+
+Generate and return a closure that, when called with a line, will
+return a hash reference containing the parsed fields, or undef if the
+parse failed. If LIST is supplied, it is interpreted as a flattened
+hash of arguments. One argument is recognised; if C<reuse_record> is a
+true value, then the closure will reuse the same hash reference each
+time it is called. The default is to allocate a new hash for each
+result.
+
+Calling this closure is significantly faster than the C<parse> method.
+
 =item names()
 
 Returns a list of field names that were extracted from the data. Such as
